﻿using System.Text;
using System.Windows;
using System.Windows.Controls;

namespace WpfTextBoxOnlyNumeric.TextBoxNumeric
{
	//====================================================
	// Описание работы классов и методов исходника на:
	// https://www.interestprograms.ru
	// Исходные коды программ и игр
	//====================================================



	public class TextBoxNumeric : TextBox
	{
		#region Открытые свойства класса

		// Свойство хранит значение по умолчанию текста элемента
		private string _defaultValue = "0";
		public string DefaultValue
		{
			get { return _defaultValue; }
			set { _defaultValue = value; }
		}

		// Открытое свойство указывающее восстанавливать ли последнее значение
		// в случае полного удаления текста элемента.
		// Восстановление происходит при потере фокуса.
		private bool _restoreDefaultValue = false;
		public bool RestoreDefaultValue
		{
			get { return _restoreDefaultValue; }
			set { _restoreDefaultValue = value; }
		}

		// Вид десятичного разделителя
		char _decimalSeparator = '.';
		public char DecimalSeparator
		{
			get { return _decimalSeparator; }
			set { _decimalSeparator = value; }
		}


		// Вид текстового поля для ввода целых чисел,
		// или чисел с плавающей запятой.
		private bool _inputDouble = false;
		public bool InputDouble
		{
			get { return _inputDouble; }
			set { _inputDouble = value; }
		}

		#endregion


		#region События, формирующие функциональность числового текстового поля

		// Уведомление об изменениях свойств зависимости элемента TextBox.
		protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
		{
			// Обязательный вызов базовой реализации.
			base.OnPropertyChanged(e);

			// Включаем работу кода только в контексте свойста Text элемента TextBox.
			if (e.Property == TextBox.TextProperty)
			{
				if (e.NewValue is string newv == true)
				{
					// Если текстовое поле не пусто и есть изменения.
					if (Text != "" && newv.Equals(e.OldValue) == false)
					{
						// Выбор фильтра для коррекции ввода символов
						if (_inputDouble == false) Text = FilterInteger(newv);
						if (_inputDouble == true) Text = FilterFloat(newv);
					}
				}
			}
		}


		protected override void OnLostFocus(RoutedEventArgs e)
		{
			// Обязательный вызов базовой реализации.
			base.OnLostFocus(e);

			// Если разрешен вывод значения по умолчанию в пустое текстовое поле.
			if (Text == "" && RestoreDefaultValue == true)
			{
				Text = _defaultValue;
			}
		}


		#endregion


		#region Фильтры ввода текста

		// Фильтр формирования корректного целого числа.
		private string FilterInteger(string inputText)
		{
			StringBuilder res = new();
			bool onlyoneBeep = true;

			for (int i = 0; i < inputText.Length; i++)
			{
				// Фильтр ввода чисел, пропускает только числа (цифры).
				if (Char.IsDigit(inputText[i]) == true)
				{
					// Добавляем символ в строку результата.
					res.Append(inputText[i]);
				}
				else
				{
					if (onlyoneBeep == true)
					{
						onlyoneBeep = false;
						Console.Beep(300, 100);
					}
				}
			}


			// Удаление первого нуля, если количество цифр больше 1.
			if (res.Length > 1)
			{
				if (res[0] == '0')
				{
					res.Remove(0, 1);
				}
			}

			return res.ToString();
		}


		// Фильтр формирования корректного числа с дробной частью
		private string FilterFloat(string inputText)
		{
			StringBuilder res = new();
			bool onlyoneBeep = true;
			bool onlyonePunctuation = true;


			for (int i = 0; i < inputText.Length; i++)
			{
				// Фильтр ввода чисел, пропускает только числа (цифры).
				if (Char.IsDigit(inputText[i]) == true)
				{
					// Добавляем символ в строку результата.
					res.Append(inputText[i]);
				}
				else if (onlyonePunctuation == true && Char.IsPunctuation(inputText[i]) == true)
				{
					// Добавляем символ в строку результата только один раз
					onlyonePunctuation = false;
					// Все виды разделителей заменяем на установленный.
					res.Append(_decimalSeparator);
				}
				else
				{
					// Недовольство звуком только один раз на всю новую строку или символ.
					if (onlyoneBeep == true)
					{
						onlyoneBeep = false;
						Console.Beep(300, 100);
					}
				}

			}


			// Удаление первого нуля, если за ним не следует разделитель.
			if (res.Length > 1)
			{
				if (res[0] == '0' && res[1] != _decimalSeparator)
				{
					res.Remove(0, 1);
				}
			}

			return res.ToString();
		}

		#endregion

	}
}
